/****************************************************************************
 * PURPOSE: To further develop Stack skills through an exploration of a plant
 *					architecture known as Bracketed Lindenmayer-Systems (L-Systems)
 * DATE:		2006 10 30
 ****************************************************************************/
import java.util.*;
import java.awt.*;
import javax.swing.*;


public class Garden extends JFrame {
  int width = 1000;
  int height = 600;
  Bed bed;

  public Garden() {
    super("Garden");
    bed = new Bed(width, height);
    getContentPane().add(bed);
    setBounds(0, 0, width, height);
    setVisible(true);
  }

  public static void main(String[] args) {
    Garden garden = new Garden();

    garden.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }

}


// inner Bed class
class Bed extends JPanel {

  LinkedList<Plant> plants = new LinkedList<Plant>();
	
  // drawing details...
  int width, height;
  int currX, currY, nextX, nextY;
  int distance;
  double delta;
  double heading;
  int n;

  String current;
	
  public Bed(int width, int height) {
    super();
    this.width = width;
    this.height = height;

    // add some plants
    // page 25, a)
    String[] productions = new String[1];

    productions[0] = "F[+F]F[-F]F";
    plants.add(new Plant("F", "F", productions, 128, 25.7, 5));

    // page 25, b)
    productions = new String[1];
    productions[0] = "F[+F]F[-F][F]";
    plants.add(new Plant("F", "F", productions, 256, 20.0, 5));

    // page 25, c)
    productions = new String[1];
    productions[0] = "FF-[-F+F+F]+[+F-F-F]";
    plants.add(new Plant("F", "F", productions, 128, 22.5, 4));

    // page 25, d)
    productions = new String[2];
    productions[0] = "F[+X]F[-X]+X";
    productions[1] = "FF";
    plants.add(new Plant("X", "XF", productions, 256, 20.0, 7));

    // page 25, e)
    productions = new String[2];
    productions[0] = "F[+X][-X]FX";
    productions[1] = "FF";
    plants.add(new Plant("X", "XF", productions, 256, 25.7, 6));

    // page 25, f)
    productions = new String[2];
    productions[0] = "F-[[X]+X]+F[+FX]-X";
    productions[1] = "FF";
    plants.add(new Plant("X", "XF", productions, 256, 22.5, 5));

    // page 25, g)
    productions = new String[2];
    productions[0] = "YFX[+Y][-Y]";
    productions[1] = "X[-FFF][+FFF]FX";
    plants.add(new Plant("Y", "YX", productions, 256, 25.7, 6));

  }

  public void paintComponent(Graphics g) {

    Graphics2D g2 = (Graphics2D) g;

    g2.setBackground(Color.WHITE);
    g2.clearRect(0, 0, width, height);

    Stack<Turtle> stack = new Stack<Turtle>();
    Turtle turtle;

    ListIterator<Plant> iterator = plants.listIterator(0);

    int item = -1;

    while (iterator.hasNext()) {
      item++;

      // Get the plant object...
			<...missing code...>

      // obtain its details...
			<...missing code...>

      // set the turtle (drawing) characteristics...
			<...missing code..>

      for (int i = 0; i < current.length(); i++) {
        switch (current.charAt(i)) {

			<...missing code...>

       }
      }
    }
  }
}


/********************************************
 * inner class
 ********************************************/
class Plant {
  String axiom, keys, definition;
  String[] productions;
  int distance;
  double delta;
  int n;

  public Plant(String axiom,
      String keys,
      String[] productions,
      int distance,
      double delta,
      int n) {
		
    // generate mature description
    for (int i = 0; i < n; i++) {
			<...missing code...>
    }
  }	

  private String getSegment(char ch) {
    int which = keys.indexOf(ch);
    if (which < 0) {
      return "" + ch;
    }
    return productions[which];
  }

  public String getDefinition() {
    return definition;
  }

  public double getDelta() {
    return delta;
  }

  public int getDistance() {
    return distance;
  }

}


/*******************************************
 * Inner Turtle class
 *******************************************/
class Turtle {
  double heading;
  int xPos, yPos;
  Color color;
 	
  public Turtle(int xPos, //
      int yPos, //
      double heading, //
      Color color) //
  {
    this.xPos = xPos;
    this.yPos = yPos;
    this.heading = heading;
    this.color = color;
  }
 	
  public int getxPos() {
    return xPos;
  }	
 	
  public int getyPos() {
    return yPos;
  }	
 	
  public double getHeading() {
    return heading;
  }	

  public Color getColor() {
    return color;
  }
}


/*******************************************
 * inner Palette class
 *******************************************/
class Palette {
  Color[] palette;
	
  public Palette() {
    Random random = new Random();

    palette = new Color[256];
			
    // assign random colors to a palette
    for (int i = 0; i < 256; i++) {
      palette[i] = new Color(random.nextInt(256), random.nextInt(256),
          random.nextInt(256));
    }
  }	

  // accessor method...
  public Color getColor(int i) {
    return palette[i];
  }
}
